rawset(_G, "cv_wickedchance", CV_RegisterVar({
	name = "wickedchance",
	defaultvalue = 10,
	flags = CV_NETVAR|CV_SAVE,
	PossibleValue = {MIN = 0, MAX = 100},
}))

rawset(_G, "cv_ignorelimit", CV_RegisterVar({
	name = "ignorelimit",
	defaultvalue = 0,
	flags = CV_NETVAR|CV_SAVE,
	PossibleValue = CV_YesNo,
}))

-- Spawn Something Wicked.
addHook("LinedefExecute", function(l, mo, s)
	if server.didsomethingwicked then return end

	for p in players.iterate do
		p.somethingwicked = 9*TICRATE
		if not p.jumpscare_type then S_StartSound(nil, sfx_somwck, p) end
	end

	local possibleSpawns = {}
	for mo in mobjs.iterate() do
		if mo.type == MT_BOSSFLYPOINT then possibleSpawns[#possibleSpawns+1] = mo end
	end

	local sp = P_RandomRange(1, #possibleSpawns)
	local spawn = possibleSpawns[sp]

	local mo = P_SpawnMobj(spawn.x, spawn.y, spawn.z, MT_SOMETHINGWICKED)

	if spawn.spawnpoint then
		mo.curwaypoint = spawn.spawnpoint.args[0]
		somethingWickedWaypoint(mo)
	else
		mo.curwaypoint = 0
		somethingWickedWaypoint(mo)
	end

	server.didsomethingwicked = true
end, "SOMETHINGWICKED")

-- Stats n stuff
addHook("PlayerThink", function(p)
	if gamemap == 780 then
		p.charability = 0
		p.charability2 = 0
		p.something_wicked_stats = true

		if p.jumpscare_type then
			p.mo.momx = 0
			p.mo.momy = 0
			p.normalspeed = 0
			p.jumpfactor = 0
			p.runspeed = 0
		else
			p.normalspeed = min(18*FRACUNIT, skins[p.skin].normalspeed)
			p.jumpfactor = min(2*FRACUNIT/3, skins[p.skin].jumpfactor)
			p.runspeed = 50*FRACUNIT
		end
	elseif p.something_wicked_stats then
		p.normalspeed = skins[p.skin].normalspeed
		p.jumpfactor = skins[p.skin].jumpfactor
		p.runspeed = skins[p.skin].runspeed
		p.charability = skins[p.skin].ability
		p.charability2 = skins[p.skin].ability2
		p.something_wicked_stats = nil
	end

	if p.somethingwicked then
		p.somethingwicked = $-1
		if p.somethingwicked == 5*TICRATE then S_StartSound(nil, sfx_umsg, p) end
		if p.somethingwicked <= 0 then p.somethingwicked = nil end
	end
end)

-- Reset all jumpscare variables.
addHook("PlayerSpawn", function(p)
	if gamemap == 780 then
		p.jumpscare_type = nil
		p.jumpscare_timer = nil
		p.jumpscare_appeartimer = nil
		p.jumpscare_rush = nil
	end
end)

-- Make something wicked spawn the next time the map loads, too!
addHook("MapChange", function(mapid)
	if not server.render_weird then hud.enable("stagetitle") end

	if server.continue_levels and server.didsomethingwicked then
		server.render_weird2 = true
		hud.disable("stagetitle")
	end

    if server and (cv_ignorelimit.value > 0 or not server.somethingwickedforce) and cv_wickedchance.value > 0 and P_RandomChance(cv_wickedchance.value*(FRACUNIT/100)) and not server.render_weird2 then
		server.render_weird = true
        server.somethingwickedforce = true
		server.continue_levels = mapid
		hud.disable("stagetitle")
    end

	server.wicked_frenzy = nil
	server.didsomethingwicked = nil
end)

-- Force Something Wicked! Also, make sure that 0-S doesn't happen twice!
addHook("PostThinkFrame", function()
	if server and server.render_weird and (leveltime > 35) and gamemap ~= 780 then
		server.somethingwickedforce = true
		G_SetCustomExitVars(780, 2)
		G_ExitLevel()
	elseif server and server.render_weird2 and (leveltime > 35) and gamemap == 780 then
		G_SetCustomExitVars(server.continue_levels, 2)
		G_ExitLevel()
		server.continue_levels = nil
    end

    if gamemap == 780 and not server.wicked_frenzy then
		if leveltime > 4 and server.render_weird then
			server.render_weird = nil
			hud.enable("stagetitle")
		end

        for sector in sectors.iterate do
            if not sector.damagetype then
                sector.damagetype = sector.lightlevel
            elseif sector.lightlevel > sector.damagetype then
                server.wicked_frenzy = true
				if not consoleplayer.jumpscare_type then S_StartSound(nil, sfx_somwck, consoleplayer) end
            end
        end
	else
		if leveltime > 4 and server and server.render_weird2 then
			server.continue_levels = nil
			server.render_weird2 = nil
			hud.enable("stagetitle")
			hud.enable("time")
			hud.enable("score")
			hud.enable("rings")
			hud.enable("lives")
		end
    end
end)

-- Disable (most) abilities
addHook("PreThinkFrame", function()
	if gamemap ~= 780 then return end

	for p in players.iterate do
		local buttons = BT_USE|BT_ATTACK|BT_FIRENORMAL|BT_CUSTOM1|BT_CUSTOM2|BT_CUSTOM3|BT_TOSSFLAG
		if not P_IsObjectOnGround(p.mo) and not (p.pflags & PF_STARTJUMP) then buttons = $ | BT_JUMP end
		p.cmd.buttons = $ & ~buttons
	end
end)

-- The message...
local w, fw, h, fh, rw, rh, frw, frh
hud.add(function(v, p, cam)
	w, fw = v.dupx()
	h, fh = v.dupy()
	rw, frw, rh, frh = v.width()/w, FixedDiv(v.width()*FRACUNIT, fw), v.height()/h, FixedDiv(v.height()*FRACUNIT, fh)

	if p.somethingwicked and p.somethingwicked < 5*TICRATE then
		v.drawString(rw/2, 160, "Something wicked this way comes...", V_INVERTMAP|V_SNAPTOTOP|V_SNAPTOLEFT|V_ALLOWLOWERCASE, "center")
		hud.disable("time")
		hud.disable("score")
		hud.disable("rings")
		hud.disable("lives")
	end
end)